//
//  Shader.fsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//


varying lowp vec2 frag_uv1;
varying lowp vec4 v_position;
varying lowp vec3 v_normal;
varying lowp float v_fog;
varying lowp vec4 colorVarying;
VARYING_MAT3(tangentSpace);
varying highp vec3 v_view_dir;

uniform sampler2D env_map;

uniform highp mat4 viewInverse;
uniform highp mat4 mMatrix;
uniform highp mat4 vpMatrix;
uniform highp mat3 normalMatrix;

uniform lowp vec3 light_dir;

uniform lowp vec4 color;
uniform lowp float ambient_mult;
uniform lowp float diffuse_mult;

F_FOG_UNIFORMS


void main() {

    lowp vec3 lightColor = vec3(1.0, 0.93, 0.80);
    lowp float gloss = 1.0;
    
    gl_FragColor = color;
    
    
    mediump vec3 normalDirection = normalize(MAKE_MAT3_FROM_VARYING(tangentSpace) * vec3(0,0,1));
    highp vec3 viewDirection = normalize(v_view_dir);
    lowp vec3 lightDirection = -normalize(light_dir);
    lowp vec3 vr = normalize(reflect(-viewDirection, normalDirection));
    highp float PI = 3.14159265358979323846264;
    //lowp vec2 lat_uv = vec2((atan(vr.x/vr.z) + PI) / 2.0*PI, (asin(vr.y) + PI*0.5)/PI);
    lowp vec2 lat_uv = vec2(0.25 + atan(vr.z, vr.x)/(2.0*PI), 0.5 + (asin(vr.y)/PI));
    
    lowp vec3 refl_color = texture2D(env_map, lat_uv).xyz;
    
    lowp float nDotVP = max(0.0, dot(normalDirection, lightDirection));
    lowp float fresnel = 1.0 - pow(dot(normalDirection, viewDirection), 2.0);
    fresnel = 0.1 + 0.9*max(0.0, fresnel);


    lowp float refl_amt = fresnel*gloss;
    
    gl_FragColor.xyz = gl_FragColor.xyz * vec3(0.9, 0.9, 1.0)*ambient_mult + gl_FragColor.xyz*nDotVP*lightColor*diffuse_mult;
    gl_FragColor.xyz = gl_FragColor.xyz*(1.3-refl_amt) + refl_color*5.0*refl_amt;
    gl_FragColor.w += length(refl_color)*refl_amt*2.0;
    
    

}
