//
//  Shader.fsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//


varying lowp vec2 frag_uv1;
varying lowp vec4 v_position;
varying lowp vec3 v_normal;
varying lowp float v_fog;
varying lowp vec4 colorVarying;
VARYING_MAT3(tangentSpace);
varying highp vec4 shadowCoord;
varying highp vec3 v_view_dir;

uniform sampler2D texture;
uniform sampler2D normalMap;
uniform SHADOW_SAMPLER_FD shadowMap;
uniform sampler2D gloss_map;
uniform sampler2D env_map;

uniform highp mat4 viewInverse;
uniform highp mat4 mMatrix;
uniform highp mat4 vpMatrix;
uniform highp mat3 normalMatrix;

uniform lowp vec3 light_dir;

uniform lowp float ambient_mult;
uniform lowp float diffuse_mult;

F_FOG_UNIFORMS


void main()
{

//    lowp float pixel_size = 1.0/2048.0;
    lowp vec3 lightColor = vec3(1.0, 0.93, 0.80);
    lowp vec4 result = texture2D(texture, frag_uv1);
    lowp vec4 spec_gloss = texture2D(gloss_map, frag_uv1);
    lowp float gloss = spec_gloss.y;
    lowp float spec = spec_gloss.x*0.5 + gloss*0.9;
    
    lowp vec3 nm = normalize(texture2D(normalMap, frag_uv1).xyz*2.0 - 1.0).xyz;
    lowp float shadowVal = 0.0;
    highp vec4 m_shadow_coord = shadowCoord;
    shadowVal = CALC_SHADOW_VAL_FD(shadowMap, shadowCoord);
    gl_FragColor = result;
    
    
    mediump vec3 normalDirection = normalize(MAKE_MAT3_FROM_VARYING(tangentSpace) * nm);
    highp vec3 viewDirection = normalize(v_view_dir);
    lowp vec3 lightDirection = -normalize(light_dir);
    lowp vec3 vr = normalize(reflect(-viewDirection, normalDirection));
    highp float PI = 3.14159265358979323846264;
    //lowp vec2 lat_uv = vec2((atan(vr.x/vr.z) + PI) / 2.0*PI, (asin(vr.y) + PI*0.5)/PI);
    lowp vec2 lat_uv = vec2(0.25 + atan(vr.z, vr.x)/(2.0*PI), 0.5 + (asin(vr.y)/PI));
    
    lowp vec3 refl_color = texture2D(env_map, lat_uv).xyz;
    
    lowp float nDotVP = max(0.0, dot(normalDirection, lightDirection));
    lowp float fresnel = 1.0 - pow(dot(normalDirection, viewDirection), 2.0);
    fresnel = 0.3 + 0.7*max(0.0, fresnel);


    lowp float refl_amt = fresnel*gloss*(0.5 + shadowVal*0.5);
    
    gl_FragColor.xyz = gl_FragColor.xyz * vec3(0.9, 0.9, 1.0)*ambient_mult + gl_FragColor.xyz*nDotVP*lightColor*diffuse_mult*shadowVal;
    gl_FragColor.xyz = gl_FragColor.xyz*(1.3-refl_amt) + pow(refl_color, vec3(0.9))*5.0*refl_amt;
    
    
    

}
