//
//  Shader.fsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//

varying lowp vec2 frag_uv1;
varying lowp vec4 v_position;
varying lowp vec3 v_normal;
varying lowp vec4 colorVarying;
VARYING_MAT3(tangentSpace);
varying highp vec4 shadowCoord;

uniform sampler2D texture;
uniform sampler2D normalMap;

uniform highp mat4 viewInverse;
uniform highp mat4 mMatrix;
uniform highp mat4 vpMatrix;
uniform highp mat3 normalMatrix;

uniform lowp float ambient_mult;
uniform lowp float diffuse_mult;


void main()
{

    mediump vec3 lightColor = vec3(0.95, 0.95, 1.0);
    mediump vec4 result = texture2D(texture, frag_uv1);
    mediump vec3 nm = normalize(texture2D(normalMap, frag_uv1).xyz*2.0 - 1.0).xyz;
    gl_FragColor = result;
    
    
    mediump vec3 normalDirection = MAKE_MAT3_FROM_VARYING(tangentSpace) * nm;
    //mediump vec3 normalDirection = normalize(normalMatrix * v_normal);
    mediump vec3 lightDirection = normalize(vec3(0.1, 1.0, 0.2));
    mediump vec3 viewDirection = normalize(vec3(viewInverse * vec4(0.0, 0.0, 0.0, 1.0) - mMatrix * v_position));
    
    mediump float nDotVP = max(0.0, dot(normalDirection, normalize(lightDirection)));
                 
    mediump vec3 specularReflection;
    if (dot(normalDirection, lightDirection) < 0.0) // light source on the wrong side?
    {
        specularReflection = vec3(0.0, 0.0, 0.0); // no specular reflection
    } else { // light source on the right side

        specularReflection = lightColor * pow(max(0.0, dot(reflect(-lightDirection, normalDirection), viewDirection)), 2.0);
    }


    
    gl_FragColor.xyz = NORMAL_MAP_LIGHTING(lightColor);
    gl_FragColor.w = 1.0;
    
    

}
