//
//  Shader.fsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//


varying mediump vec2 frag_uv1;
varying lowp float v_fog;
VARYING_MAT3(tangentSpace);
varying highp vec4 v_shadow_coord;
varying lowp vec3 v_bloom_dir;
varying highp float v_bone;

uniform sampler2D diffuse;
uniform sampler2D normal_map;
uniform sampler2D color_modulation;
uniform sampler2D shadowmap;

uniform lowp vec3 light_dir;
uniform lowp vec3 skin_color;
uniform lowp float burn;
uniform lowp float ambient_mult;
uniform lowp vec4 bloom_color;

F_FOG_UNIFORMS

void main()
{
    

//    gl_FragColor = vec4(1,1,1,1) * v_bone/10.0;
    
    
    lowp float shadow_val = texture2DProj(shadowmap, v_shadow_coord).x;
    lowp float color_mod = texture2D(color_modulation, frag_uv1).x;
    gl_FragColor = texture2D(diffuse, frag_uv1);
    lowp vec3 nm = normalize(texture2D(normal_map, frag_uv1).xyz*2.0 - 1.0).xyz;

    gl_FragColor.xyz = (gl_FragColor.xyz*(1.0-color_mod) + gl_FragColor.xyz*(skin_color * color_mod))*burn;

    lowp vec3 lightDirection = normalize(light_dir);
    
    highp vec3 normalDirection = MAKE_MAT3_FROM_VARYING(tangentSpace) * nm;
    highp float nDotVP = max(0.0, dot(normalDirection, lightDirection));
    // no NOT normalize bloom dir as we encode it's intensity in it's length
    highp float bloomEffect = max(0.0, dot(normalDirection, v_bloom_dir));
    
    gl_FragColor = gl_FragColor*(nDotVP*2.0*shadow_val + 0.5*ambient_mult) + bloomEffect*bloom_color*9.0;
    ADD_FOG(v_fog, fog_color);
}
